"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const webpack_merge_1 = __importDefault(require("webpack-merge"));
const plugins_1 = require("@ewizardjs/builder-utils/plugins");
const webpack_config_1 = __importDefault(require("../bubbles/webpack.config"));
const content_type_loader_1 = require("../../loaders/content-type-loader");
const ResetStatePlugin_1 = __importDefault(require("../../plugins/ResetStatePlugin"));
const InjectCommonChunksPlugin_1 = __importDefault(require("../../plugins/InjectCommonChunksPlugin"));
const get_entry_common_loader_1 = __importDefault(require("../../get-entry-common-loader"));
const fs_1 = require("fs");
const rules_1 = require("@ewizardjs/builder-utils/rules");
const settings_1 = require("@ewizardjs/settings");
const { readFile, readdir } = fs_1.promises;
exports.default = async (options) => {
    const baseConfig = await webpack_config_1.default(options);
    const { presentationPath, systemSettings } = options;
    const currentThemeName = await settings_1.getCurrentTheme(presentationPath);
    const themesList = await getThemesList(presentationPath, systemSettings);
    const uniqueBlocksList = await contentHasDynamicBlocksLoader(presentationPath, systemSettings) ?
        getCurrentBlocks(presentationPath, systemSettings) :
        [];
    const cornConfig = {
        entry: {
            app: [require.resolve('../../../assets/entry-common'), options.appEntryPath],
            index: options.indexEntryPath,
            editor: require.resolve('../../../assets/entry-editor'),
        },
        optimization: {
            splitChunks: {
                hidePathInfo: true,
                cacheGroups: {
                    common: {
                        name: 'common',
                        chunks: 'all',
                        minSize: 1310720,
                        maxSize: 5242880,
                        minChunks: 2,
                    },
                },
            },
        },
        module: {
            rules: [
                ...content_type_loader_1.getContentTypeLoader({
                    include: /node_modules/,
                }),
                get_entry_common_loader_1.default(options.presentationPath),
                rules_1.currentTheme({
                    options: {
                        currentTheme: currentThemeName,
                        presentationPath,
                        themes: themesList,
                    },
                }),
                rules_1.currentBlocks({
                    options: {
                        blocksList: uniqueBlocksList,
                    },
                }),
            ],
        },
        output: {
            library: ['Content', '[name]'],
            jsonpFunction: 'ewizardjsJsonp',
        },
        externals: {
            './app': {
                commonjs: ['Content', 'app'],
                commonjs2: ['Content', 'app'],
                root: ['Content', 'app'],
            },
        },
        plugins: [
            new ResetStatePlugin_1.default(options.presentationPath),
            new InjectCommonChunksPlugin_1.default({
                live: options.live,
                watch: options.watch,
            }),
            plugins_1.cleanDestinationFolder(options.presentationPath, path_1.join(options.presentationPath, 'build', 'dev')),
        ],
    };
    return webpack_merge_1.default.strategy({
        entry: 'replace',
        externals: 'replace',
        'output.library': 'replace',
        'module.rules': 'prepend',
    })(baseConfig, cornConfig);
};
async function contentHasDynamicBlocksLoader(presentationPath, systemSettings) {
    const vueApp = path_1.join(presentationPath, systemSettings.path.rootComponent);
    const fileContent = await readFile(vueApp, 'utf8');
    if (fileContent) {
        const dynamicBlockComponentName = /(<DynamicBlocksLoader|<dynamic-blocks-loader)/i;
        return dynamicBlockComponentName.test(fileContent);
    }
    return false;
}
function getCurrentBlocks(presentationPath, systemSettings) {
    var _a, _b;
    const blocksJson = require(path_1.resolve(presentationPath, systemSettings.path.blocksManifest));
    const blockThemesJson = require(path_1.resolve(presentationPath, systemSettings.path.blocksThemes));
    const blocksComponents = blocksJson.components.map(block => ({ path: block.path, id: block.id }));
    const blocksByThemes = {};
    const blocksForThemes = [];
    for (const theme in blockThemesJson['themes']) {
        const themeBlocks = (_b = (_a = blockThemesJson['themes']) === null || _a === void 0 ? void 0 : _a[theme]) === null || _b === void 0 ? void 0 : _b.blocks;
        const themeBlocksIds = themeBlocks.map(block => ({ theme, path: block.path, id: block.id }));
        blocksByThemes[theme] = themeBlocksIds;
        blocksForThemes.push(...themeBlocksIds);
    }
    const allBlocksList = [...blocksForThemes, ...blocksComponents];
    const uniqueBlocksList = [];
    allBlocksList.filter((block) => {
        if (!uniqueBlocksList.find(elem => elem.id === block.id && block.theme === elem.theme)) {
            uniqueBlocksList.push(block);
            return true;
        }
        return false;
    });
    return uniqueBlocksList;
}
async function getThemesList(presentationPath, systemSettings) {
    const themesJsonPath = path_1.resolve(presentationPath, systemSettings.path.themesManifest);
    try {
        const themesJson = require(themesJsonPath);
        return themesJson.themes.map((theme) => theme.id);
    }
    catch (error) {
        const themesDir = path_1.resolve(presentationPath, systemSettings.path.themes);
        return await readdir(themesDir);
    }
}
//# sourceMappingURL=webpack.config.js.map